@@ -45,6 +45,37 @@ Glib::ustring Document::get_file_uri() const
   return m_file_uri;
 }
 
+Glib::ustring Document::get_file_uri_with_extension(const Glib::ustring& uri)
+{
+  Glib::ustring result = uri;
+  
+  //Enforce file extension:
+  if(!m_file_extension.empty())  //If there is an extension to enforce.
+  {
+    bool bAddExt = false;
+    Glib::ustring strExt = "." + get_file_extension();
+
+    if(result.size() < strExt.size()) //It can't have the ext already if it's not long enough.
+    {
+      bAddExt = true; //It isn't there already.
+    }
+    else
+    {
+      Glib::ustring strEnd = result.substr(result.size() - strExt.size());
+      if(strEnd != strExt) //If it doesn't already have the extension
+        bAddExt = true;
+    }
+
+    //Add extension if necessay.
+    if(bAddExt)
+      result += strExt;
+
+    //Note that this does not replace existing extensions, so it could be e.g. 'something.blah.theext'
+  }
+
+  return result;
+}
+
 void Document::set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension /* = false */)
 {
   if(file_uri != m_file_uri)
@@ -54,30 +85,7 @@ void Document::set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExte
 
   //Enfore file extension:
   if(bEnforceFileExtension)
-  {
-    if(m_strFileExtension.size() != 0)  //If there is an extension to enforce.
-    {
-      bool bAddExt = false;
-      Glib::ustring strExt = "." + get_file_extension();
-      
-      if(m_file_uri.size() < strExt.size()) //It can't have the ext already if it's not long enough.
-      {
-        bAddExt = true; //It isn't there already.
-      }
-      else
-      {
-        Glib::ustring strEnd = m_file_uri.substr(m_file_uri.size()-strExt.size());
-        if(strEnd != strExt) //If it doesn't already have the extension
-          bAddExt = true;
-      }
- 
-      //Add extension if necessay.
-      if(bAddExt)
-        m_file_uri += strExt;
-        
-      //Note that this does not replace existing extensions, so it could be e.g. 'something.blah.theext'
-    }
-  }
+    m_file_uri = get_file_uri_with_extension(m_file_uri);
 }
 
 void Document::set_contents(const Glib::ustring& strVal)
@@ -219,10 +227,11 @@ bool Document::write_to_disk()
     the second to last tells GnomeVFS that its ok if the file already exists, and just open it */
 
     Gnome::Vfs::Handle write_handle;
-      
+
     try
     {
       //0660 means "this user and his group can read and write this non-executable file".
+      //The 0 prefix means that this is octal.
       write_handle.create(m_file_uri, Gnome::Vfs::OPEN_WRITE, false, 0660 /* leading zero means octal */);
     }
     catch(const Gnome::Vfs::exception& ex)
@@ -241,7 +250,7 @@ bool Document::write_to_disk()
       // If the operation was not successful, print the error and abort
       return false; //print_error(ex, output_uri_string);
     }
-  
+
     return true; //Success. (At doing nothing, because nothing needed to be done.)
   }
   else
@@ -250,32 +259,32 @@ bool Document::write_to_disk()
 
 Glib::ustring Document::get_name() const
 {
-  return util_file_uri_get_name(m_file_uri);
+  return util_file_uri_get_name(m_file_uri, m_file_extension);
 }
 
-Glib::ustring Document::util_file_uri_get_name(const Glib::ustring& file_uri)
+Glib::ustring Document::util_file_uri_get_name(const Glib::ustring& file_uri, const Glib::ustring& file_extension)
 {
-  Glib::ustring strResult;
+  Glib::ustring strResult = Glib::filename_display_basename(file_uri);
 
-  if(file_uri.size())
+  //Remove the file extension:
+  //TODO: Maybe filename_display_basename() should do this.
+  if(!strResult.empty() && !file_extension.empty())
   {
-    //Find position of last '/' character.
-    Glib::ustring::size_type posLastSlash = file_uri.find_last_of("/");
-    if(posLastSlash == Glib::ustring::npos)
+    const Glib::ustring strExt = "." + file_extension;
+
+    if(strResult.size() >= file_extension.size()) //It can't have the ext already if it's not long enough.
     {
-      //No '/' found. The file must be in the current or top-level directory.
-      strResult = file_uri;
-    }
-    else
-    {
-      //'/' found. The filename is the text after this: e.g. sub/sub/filename:
-      strResult = file_uri.substr(posLastSlash+1);
+      Glib::ustring strEnd = strResult.substr(strResult.size() - strExt.size());
+      if(strEnd == strExt) //If it has the extension
+      {
+        strResult = strResult.substr(0, strResult.size() - strExt.size());
+      }
     }
   }
 
   //Show untitled explicitly:
   //Also happens for file_uris with path but no name. e.g. /sub/sub/, which shouldn't happen.
-  if(strResult.size() == 0)
+  if(strResult.empty())
     strResult = gettext("Untitled");
 
   return strResult;
@@ -316,7 +325,7 @@ bool Document::get_read_only() const
       }
     }
   }
-  
+
   return m_bReadOnly;
 }
 
@@ -334,18 +343,18 @@ void Document::set_is_new(bool bVal)
 {
   if(bVal)
     set_modified(false); //can't be modified if it is new.
-    
-  m_bIsNew = bVal; 
+
+  m_bIsNew = bVal;
 }
 
 void Document::set_file_extension(const Glib::ustring& strVal)
 {
-  m_strFileExtension = strVal;
+  m_file_extension = strVal;
 }
 
 Glib::ustring Document::get_file_extension() const
 {
-  return m_strFileExtension; //TODO: get it from the mime-type system?
+  return m_file_extension; //TODO: get it from the mime-type system?
 }
 
 Document::type_signal_modified& Document::signal_modified()
